;; winnt.el --- Lisp routines for Windows NT.
;; Copyright (C) 1994 Free Software Foundation, Inc.

;; Author: Geoff Voelker (voelker@cs.washington.edu)

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; (August 12, 1993)
;; Created.

;; (November 21, 1994)
;; [C-M-backspace] defined.
;; mode-line-format defined to show buffer file type.
;; audio bell initialized.

;;; Code:

; patch 97.1.25  by K.Fujii
(defvar win32-shells '("cmd" "command"))
(defvar unixy-shells '("sh" "bash" "tcsh"))
(defvar unixy-tools '("ls" "cp" "mv" "cat"
		      "make"
		      "grep" "awk" "gawk" "sed"
		      "tar" "gzip" "gunzip" "compress"
		      "diff" "patch"
		      "yacc" "bison" "lex" "flex"
		      "hexl" "coco" "ctags" "etags" "tcp" "wakup"
		      "emacsclient" "emacsserver" "m2ps"))
; end of patch

;; Map delete and backspace
(define-key function-key-map [backspace] "\177")
(define-key function-key-map [delete] "\C-d")
(define-key function-key-map [M-backspace] [?\M-\177])
(define-key function-key-map [C-M-backspace] [\C-\M-delete])

;; patch 96.5.5  by K.Fujii
;(or (boundp 'MULE)
;; Show file type (text or binary) on modeline
;(setq-default mode-line-format
;  (list (purecopy "")
;   'mode-line-modified
;   'mode-line-buffer-identification
;   (purecopy "   ")
;   'global-mode-string
;   (purecopy "   %[(")
;   (purecopy "%t:")
;   'mode-name 'mode-line-process 'minor-mode-alist
;   (purecopy "%n")
;   (purecopy ")%]--")
;   (purecopy '(line-number-mode "L:%l--"))
;   (purecopy '(column-number-mode "C:%c--")) ;; 96.5.5 by K.Fujii
;   (purecopy '(-3 . "%p"))
;   (purecopy "-%-")))
;)
;; end of patch

;; Ignore case on file-name completion
;(setq completion-ignore-case t)

;; The cmd.exe shell uses the "/c" switch instead of the "-c" switch
;; for executing its command line argument (from simple.el).
(setq shell-command-switch "-c")

(setq explicit-cmd-args nil)
;; 95.9.11 by himi, 97.2.12  by K.Fujii
(defvar argument-conversion-slash-to-invert-slash-convert 'auto
  "When this variable is t, the argument conversion in
process functions, ('/' -> '\\'), is effective.")
(defvar argument-conversion-hyphen-to-slash-convert 'auto
  "When this variable is t, the argument conversion in
process functions, ('-' -> '/'), is effective.")
(defvar argument-conversion-quote-enclosed-not-convert 'auto
  "When this variable is t, the part enclosed
with quotes (single or double) in the argument in process functions
is not converted.")
(defvar argument-conversion-escape-char-double-invert-slash 'auto
  "When this variable is t, '\\' in the argument in
process functions is not regarded as an escape character.")
(defvar process-program-unix-to-dos-conversion nil
  "When this variable is t, a program argument given in
process functions is converted from unix form to dos form.")

;; Use ";" instead of ":" as a path separator (from files.el).
(setq path-separator ";")

;; Set the null device (for compile.el).
(setq grep-null-device "NUL:")

;; Set the grep regexp to match entries with drive letters.
(setq grep-regexp-alist
  '(("^\\(\\([a-zA-Z]:\\)?[^:( \t\n]+\\)[:( \t]+\\([0-9]+\\)[:) \t]" 1 3)))

;; patch 96.5.5  by K.Fujii
;(or (boundp 'MULE)
;    (progn
;
;; Taken from dos-fn.el ... don't want all that's in the file, maybe
;; separate it out someday.
;
;(defvar file-name-buffer-file-type-alist
;  '(
;    ("[:/].*config.sys$" . nil)		; config.sys text
;    ("\\.elc$" . t)			; emacs stuff
;    ("\\.\\(obj\\|exe\\|com\\|lib\\|sys\\|chk\\|out\\|bin\\|ico\\|pif\\)$" . t)
;					; MS-Dos stuff
;    ("\\.\\(arc\\|zip\\|pak\\|lzh\\|zoo\\)$" . t)
;					; Packers
;    ("\\.\\(a\\|o\\|tar\\|z\\|gz\\|taz\\)$" . t)
;					; Unix stuff
;    ("\\.tp[ulpw]$" . t)
;					; Borland Pascal stuff
;    ("[:/]tags$" . t)
;					; Emacs TAGS file
;    )
;  "*Alist for distinguishing text files from binary files.
;Each element has the form (REGEXP . TYPE), where REGEXP is matched
;against the file name, and TYPE is nil for text, t for binary.")
;
;(defun find-buffer-file-type (filename)
;  (let ((alist file-name-buffer-file-type-alist)
;	(found nil)
;	(code nil))
;    (let ((case-fold-search t))
;      (setq filename (file-name-sans-versions filename))
;      (while (and (not found) alist)
;	(if (string-match (car (car alist)) filename)
;	    (setq code (cdr (car alist))
;		  found t))
;	(setq alist (cdr alist))))
;    (if found
;	(cond((memq code '(nil t)) code)
;	     ((and (symbolp code) (fboundp code))
;	      (funcall code filename)))
;      default-buffer-file-type)))
;
;(defun find-file-binary (filename) 
;  "Visit file FILENAME and treat it as binary."
;  (interactive "FFind file binary: ")
;  (let ((file-name-buffer-file-type-alist '(("" . t))))
;    (find-file filename)))
;
;(defun find-file-text (filename) 
;  "Visit file FILENAME and treat it as a text file."
;  (interactive "FFind file text: ")
;  (let ((file-name-buffer-file-type-alist '(("" . nil))))
;    (find-file filename)))
;
;(defun find-file-not-found-set-buffer-file-type ()
;  (save-excursion
;    (set-buffer (current-buffer))
;    (setq buffer-file-type (find-buffer-file-type (buffer-file-name))))
;  nil)
;
;;; To set the default file type on new files.
;(add-hook 'find-file-not-found-hooks 'find-file-not-found-set-buffer-file-type)
;
;;; For using attached Unix filesystems.
;(defun save-to-unix-hook ()
;  (save-excursion
;    (setq buffer-file-type t))
;  nil)
;
;(defun revert-from-unix-hook ()
;  (save-excursion
;    (setq buffer-file-type (find-buffer-file-type (buffer-file-name))))
;  nil)
;
;; Really should provide this capability at the drive letter granularity.
;(defun using-unix-filesystems (flag)
;  (if flag
;      (progn
;	(add-hook 'write-file-hooks 'save-to-unix-hook)
;	(add-hook 'write-contents-hooks 'save-to-unix-hook)
;	(add-hook 'after-save-hook 'revert-from-unix-hook))
;    (progn
;      (remove-hook 'write-file-hooks 'save-to-unix-hook)
;      (remove-hook 'write-contents-hooks 'save-to-unix-hook)
;      (remove-hook 'after-save-hook 'revert-from-unix-hook))))
;))
;; end of patch 

;;; Fix interface to (X-specific) mouse.el
(defun x-set-selection (type data)
  (or type (setq type 'PRIMARY))
  (put 'x-selections type data))

(defun x-get-selection (&optional type data-type)
  (or type (setq type 'PRIMARY))
  (get 'x-selections type))

(fmakunbound 'font-menu-add-default)
(global-unset-key [C-down-mouse-1])
(global-unset-key [C-down-mouse-2])
(global-unset-key [C-down-mouse-3])

;;; Set to a system sound if you want a fancy bell.
(set-message-beep nil)

(fset 'suspend-emacs (function (lambda () (interactive) (minimize-window))))

;;; win32 system
;(defun set-win32-system-coding-system (coding-system)
;  (interactive "zSystem-coding-system: ")
;  (check-coding-system coding-system)
;  (setq win32-system-coding-system coding-system))

;;; Clipboard
(defun set-clipboard-coding-system (coding-system)
  (interactive "zClipboard-coding-system: ")
  (check-coding-system coding-system)
  (setq win32-clipboard-coding-system coding-system))

(defun copy-region-to-clipboard (begin end &optional coding-system)
  (interactive "r")
  (win32-set-clipboard-data (buffer-substring begin end) coding-system))

(defun append-region-to-clipboard (begin end &optional coding-system)
  (interactive "r")
  (win32-set-clipboard-data
   (concat (win32-get-clipboard-data coding-system)
	   (buffer-substring begin end)) coding-system))

(defun kill-region-to-clipboard (begin end &optional coding-system)
  (interactive "r")
  (win32-set-clipboard-data (buffer-substring begin end) coding-system)
  (delete-region begin end))

(defun yank-from-clipboard (&optional coding-system)
  (interactive)
  (insert (win32-get-clipboard-data coding-system)))

;; patch 97.5.20  by K.Fujii
(defun win32-platform-id ()
  (nth 3 (win32-os-version-info)))

(defun win32-user-language-abbrev-name ()
  "Return user default language abbrev name symbol."
  (car (win32-user-language-info)))

(defun win32-user-language-full-name ()
  "Return user default language full name symbol."
  (nth 1 (win32-user-language-info)))

(defun win32-user-countory-code ()
  "Return user countory code."
  (nth 2 (win32-user-language-info)))

(defun win32-user-codepage ()
  "Return user default codepage."
  (nth 3 (win32-user-language-info)))
;; end of patch

;; patch 97.11.25  by K.Fujii
(defun win32-font-exist-p (face-name)
  (or win32-system-fonts-list
      (win32-list-system-fonts))
  (let ((font-list win32-system-fonts-list)
	exist)
    (while (and font-list
		(null exist))
      (setq exist (string= face-name (car (car font-list))))
      (setq font-list (cdr font-list)))
    exist))
;; end of patch

;; patch 97.5.4  by K.Fujii
(defvar *win32-default-font-size* -16)
;; default-fonts.
(defvar default-fonts-12
  '(((lc-ascii . ("Courier New" (0 0 0 0) (-11 -11 -10 -10) ansi 49)) "default")))
(defvar default-fonts-14
  '(((lc-ascii . ("Courier New" (0 0 0 0) (-13 -13 -11 -11) ansi 49)) "default")))
(defvar default-fonts-16
  '(((lc-ascii . ("Courier New" (0 0 0 0) (-15 -15 -13 -13) ansi 49)) "default")))

(defconst win32-lang-name-to-lang-env-alist
  '((heb . (*iso-8859-8*  . hebrew))    ; Hebrew
    (tha . (*tis620*      . thai))      ; Thai
    (vit . (*viscii*      . viet))      ; Vetnamese
    (cat . (*iso-8859-1*  . europian))  ; Catalan
    (dan . (*iso-8859-1*  . europian))	; Danish
    (nld . (*iso-8859-1*  . europian))	; Dutch
    (nlb . (*iso-8859-1*  . europian))
    (fin . (*iso-8859-1*  . europian))	; Finnish
    (fra . (*iso-8859-1*  . europian))	; French
    (frb . (*iso-8859-1*  . europian))
    (frc . (*iso-8859-1*  . europian))
    (frs . (*iso-8859-1*  . europian))
    (frl . (*iso-8859-1*  . euripoan))
    (dea . (*iso-8859-1*  . europian))	; German
    (dec . (*iso-8859-1*  . europian))
    (del . (*iso-8859-1*  . europian))
    (des . (*iso-8859-1*  . europian))
    (deu . (*iso-8859-1*  . europian))
    (ita . (*iso-8859-1*  . europian))	; Italian
    (its . (*iso-8859-1*  . europian))
    (ptb . (*iso-8859-1*  . europian))	; Portuguese
    (ptg . (*iso-8859-1*  . europian))
    (esp . (*iso-8859-1*  . europian))	; Spanish
    (esm . (*iso-8859-1*  . europian))
    (esn . (*iso-8859-1*  . europian))
    (esg . (*iso-8859-1*  . europian))
    (esc . (*iso-8859-1*  . europian))
    (esa . (*iso-8859-1*  . europian))
    (esd . (*iso-8859-1*  . europian))
    (esv . (*iso-8859-1*  . europian))
    (eso . (*iso-8859-1*  . europian))
    (ess . (*iso-8859-1*  . europian))
    (esf . (*iso-8859-1*  . europian))
    (esl . (*iso-8859-1*  . europian))
    (esy . (*iso-8859-1*  . europian))
    (esz . (*iso-8859-1*  . europian))
    (esb . (*iso-8859-1*  . europian))
    (ese . (*iso-8859-1*  . europian))
    (esh . (*iso-8859-1*  . europian))
    (esi . (*iso-8859-1*  . europian))
    (esu . (*iso-8859-1*  . euripoan))
    (sve . (*iso-8859-1*  . europian))	; Swedish
    (svf . (*iso-8859-1*  . europian))
    (isl . (*iso-8859-1*  . europian))	; Icelandic
    (nor . (*iso-8859-1*  . europian))	; Norwegian
    (non . (*iso-8859-1*  . europian))
    (csy . (*win-latin2*  . win-latin2)); Czech
    (sky . (*win-latin2*  . win-latin2)); Slovene
    (hun . (*win-latin2*  . win-latin2)); Hungarian
    (plk . (*win-latin2*  . win-latin2)); Polish
    (srl . (*win-latin2*  . win-latin2)); Servian (Latin)
    (hrv . (*win-latin2*  . win-latin2)); Croatian
    (lth . (*iso-8859-4*  . europian))  ; Lithuanian
    (lvi . (*iso-8859-4*  . europian))  ; Latvian
    (eti . (*iso-8859-4*  . europian))  ; Estonian
    (trk . (*iso-8859-9*  . europian))	; Turkish
    (ell . (*iso-8859-7*  . greek))	; Greek
    (rus . (*win-cyrillic* . win-cyrillic)) ; Russian
    (ukr . (*win-cyrillic* . win-cyrillic)) ; Ukrainian
    (srb . (*win-cyrillic* . win-cyrillic)) ; Selvian (Cyrillic)
    (bgr . (*win-cyrillic* . win-cyrillic)) ; Bulgarian
    (jpn . (*sjis*        . japanese))	; Japanese
    (kor . (*euc-korea*   . korean))	; Korean
    (koj . (*iso-2022-kr* . korean))    ; Korian(Johab)
    (chs . (*euc-china*   . chinese))	; Chinese(prc)
    (cht . (*big5*        . chinese))	; Chinese(Taiwan)
    (zhi . (*big5*        . chinese))   ; Chinese(Singapore)
    (zhh . (*big5*        . chinese))   ; Chinese(Hong Kong)
    ))

(defun win32-setup-language-environment ()
  "Setup User Language Environment and Return Alist of
'(coding-system . language-environment-symbol)."
  (let* ((env-alist (assq win32-user-language-abbrev-name win32-lang-name-to-lang-env-alist))
	 env coding coding-dos)
    (and (consp env-alist)
	 (consp (cdr env-alist))
	 (setq env (cdr (cdr env-alist))
	       coding (car (cdr env-alist))))
    (cond ((eq env 'win-latin2)
	   (require 'easteurope))
	  ((eq env 'win-cyrillic)
	   (require 'cyrillic))
	  (t nil))
    (and env
	 (set-primary-environment env))
    (if (null coding)
	nil
      (setq coding-dos (intern (concat (symbol-name coding) "dos")))
      (and (fboundp 'set-pathname-coding-system)
	   (set-pathname-coding-system coding))
      (set-clipboard-coding-system coding-dos)
      (set-win32-system-coding-system coding-dos)
      ;; for Korean, Big5 and Turckish.
      (cond ((eq coding '*euc-korea*)
	     (set-keyboard-coding-system *euc-korea*))
	    ((memq coding '(*gig5* *iso*8859-9*))
	     (set-default-file-coding-system coding-dos)
	     (setq keyboard-coding-system coding)
	     (if (eq coding '*big5*) ;; for BIG5
		 (set-coding-priority
		  '(*code-category-big5*
		    *code-category-iso-8-2*
		    *code-category-iso-8-1*))
	       ;; for Turkish (latin-5)
	       (setq *coding-category-iso-8-1* '*iso-8859-9*)
	       (setq-default quail-current-package
			     (assoc "latin-5" quail-package-alist))))))
    (cdr env-alist)))

(defvar win32-IMEfont-alist nil)
    
(defun win32-set-fontset-font (fontset lc font-name log-font &optional coding-system)
  (let* ((name (nth 0 log-font))
	 (width (nth 1 log-font))
	 (height (nth 2 log-font))
	 (charset (nth 3 log-font))
	 (pitch-and-family (nth 4 log-font))
	 (property 0)
	 (coding  coding-system)
	 (width-half (if (or (string-match "Mule Arabic2 ETL MBF[0-9]+" name)
			     (string-match "Ethio-[1-2] ETL[0-9]+" name))
			 t nil))
	 (height-no-modify (if (and (eq lc 'lc-thai)
				    (not (or (string-match "Tahi Pen sMTF" name)
					     (string-match "Tis620 ETL[0-9]+" name))))
			       t nil))
	 (encoding (if (or (string-match "Mule Arabic[0-2] ETL MBF[0-9]+" name)
			   (string-match "Ethio-[1-2] ETL[0-9]+" name))
		       nil t)))
    (or (non-nil-coding-system-p coding)
	(setq coding
	      (cond ((memq lc '(lc-kana lc-roman lc-jpold lc-jp))
		     *sjis*)
		    ((eq lc 'lc-jp2)
		     *euc-japan*)
		    ((memq lc '(lc-big5-1 lc-big5-2))
		     *big5*)
		    ((eq lc 'lc-cn)
		     *euc-china*)
		    ((eq lc 'lc-kr)
		     (if (= win32-user-code-page 1361)
			 *iso-2022-kr*
		       *euc-korea*))
		    (t
		     nil))))
    (win32-add-font font-name lc coding fontset width-half height-no-modify encoding)
    (if (integerp width)
	(setq width (make-list 4 width))
      (and (null width)
	   (setq width (make-list 4 0))))
    (and (null height)
	 (let ((height1 (if (string-match "-?[0-9]+" fontset)
			    (string-to-int (substring fontset
						      (match-beginning 0)
						      (match-end 0))))))
	   (setq height (cond ((or (null height1)
				   (zerop height1))
			       *win32-default-font-size*)
			      ((natnump height1)
			       (- height1))
			      (t
			       height1)))))
    (if (integerp height)
	(setq height (make-list 4 height))
      (and (listp height)
	   (= (length height) 2)
	   (setq height (append (make-list 2 (car height))
				(make-list 2 (car (cdr height)))))))
    (while (< property 4)
      (win32-change-font-property font-name property
				  (list name
					(nth property width)
					(nth property height)
					(if (or (= property 1)
						(= property 3))
					    700 400)
					0
					(if (> property 1) t nil)
					nil
					charset
					1
					3
					pitch-and-family)
				  fontset)
      (setq property (1+ property)))
    ;; for IME fontt.
    (if (eq coding keyboard-coding-system)
	(if (> (length height) 4)
	    (win32-change-font-property font-name 4
					(list name
					      (if (> (length width) 4)
						  (nth 4 width)
						0)
					      (nth 4 height)
					      400 0 nil nil charset 3 2 pitch-and-family)
					fontset)
	  (let ((temp-alist (list (cons fontset font-name))))
	    (if (null win32-IMEfont-alist)
		(setq win32-IMEfont-alist temp-alist)
	      (nconc win32-IMEfont-alist temp-alist)))
	  ))))

(defconst win32-lc-char-to-lf-charset-alist
  '((lc-ltn2 . EastEurope) (lc-ltn4 . Baltic)
    (lc-thai . Thai) (lc-grk  . Greek)
    (lc-crl . Cyrillic) (lc-ltn5 . Turkish)
    (lc-vn-1 . Vietnamese) (lc-vn-2 . Vietnamese)
    (lc-hbw . Hebrew) (lc-kana . Japanese)
    (lc-roman . Japanese) (lc-jpold . Japanese)
    (lc-jp . Japanese) (lc-jp2   . Japanese)
;    (lc-kr . Korean)
    (lc-cn . gb2312)
    (lc-big5-1 . big5) (lc-big5-2 . big5)
    ))

(define-ccl-program ccl-win32-vn-2-viscii
  '(0
     ((r1 = r1
       [  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
          0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
	  0 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143
	144 145 146   0   0 147 150 151 152   0   0   0   0 180 149   0
	  0   0   0   0   0   0   34 37   0   0   0   0   0   0   0 159
	  0 186   0   0   0 128  52 187 188   0   0  57  62   0 179 191
	192 193 194 195 196 197 255  38 200 201 202 203 204 205 206 155
	208 185 210 211 212 160 153 154 158 217 218 157 156 221 148   0
	])))
  "CCL program to convert chars of lc-vn-2 to Mule VISCII TrueType font")

(defun win32-new-fontset (fontset-name font-list &optional size)
  "Make new fontset and setup fonts.
FONTSET-NAME arg is fontset-name string.
FONT-LIST arg is list of font data.
 (((lc-char . (\"System Font Name\" width height char-set pich-and-famyly)) \"font-name\")
  (.....)
 )
And Optional args is SIZE.
"
  (let* ((height (cond ((integerp size) size)
		       ((string-match "[0-9]+" fontset-name)
			(string-to-int (substring fontset-name
						  (match-beginning 0)
						  (match-end 0))))
		       (t
			*win32-default-font-size*)))
	 (width (/ height 2))
	 (tmp-logfont (make-list 5 nil))
	 (default-logfont (cdr (car (car font-list))))
	 (default-sys-font (nth 0 default-logfont))
	 (default-width (nth 1 default-logfont))
	 (default-height (nth 2 default-logfont)))
    (win32-add-fontset fontset-name width height 0 0)
    (while font-list
      (let* ((font (car font-list))
	     (font-name (car (cdr font)))
	     (lc-char (car (car font)))
	     (logfont (cdr (car font)))
	     (this-sys-font (nth 0 logfont))
	     (this-width (nth 1 logfont))
	     (this-height (nth 2 logfont))
	     (this-lf-charset (nth 3 logfont))
	     (this-pitch-and-family (nth 4 logfont))
	     (lc-group (cond ((memq lc-char '(lc-kana lc-roman lc-jpold lc-jp))
			      '(lc-kana lc-roman lc-jpold lc-jp))
			     ((memq lc-char '(lc-big5-1 lc-big5-2))
			      '(lc-big5-1 lc-big5-2))
			     ((memq lc-char '(lc-vn-1 lc-vn-2))
			      '(lc-vn-1 lc-vn-2))
			     (t
			      nil))))
	;; modify fontname.
	(if (not (stringp font-name))
	    (let ((lc-name (symbol-name lc-char)))
	      (setq font-name
		    (if (eq lc-char 'lc-ascii) "default"
		      (concat (substring lc-name 3 (length lc-name))
			      "-" (int-to-string height))))))
	;; modify System Fontname
	(setcar tmp-logfont (if this-sys-font
				this-sys-font
			      default-sys-font))
	;; modify width.
	(setcar (nthcdr 1 tmp-logfont)
		(if this-width
		    this-width
		  default-width))
	;; modify height.
	(setcar (nthcdr 2 tmp-logfont)
		(if this-height
		    this-height
		  default-height))
	;; modify lf-charset.
	(if this-lf-charset
	    (setcar (nthcdr 3 tmp-logfont) this-lf-charset)
	  (let ((tmp-facename (car tmp-logfont)))
	           ;;;  For ANSI_CHARSET FONTS.
	           ;;; for Mule TrurType Fonts.
	    (cond ((string-match " MTF$" tmp-facename)
		   (setcar (nthcdr 3 tmp-logfont) 'ansi)
		   (or this-pitch-and-family
		       (setq this-pitch-and-family 34)))
		  ;;; for Mule Bitmap Fonts or
		  ;;;     Bitmap Fonts making from ETL BDF.
		  ((or (string-match " MBF[0-9]+$" tmp-facename)
		       (string-match " ETL[0-9]+$" tmp-facename))
		   (or this-pitch-and-family
		       (setq this-pitch-and-family 17))
		   (setcar (nthcdr 3 tmp-logfont) 'ansi) )
		  ;;;  For SYMBOL_CHARSET Fonts.
		  ;;; for Mule trueType Fonts.
		  ((string-match " sMTF$" tmp-facename)
		   (setcar (nthcdr 3 tmp-logfont) 'symbol)
		   (or this-pitch-and-family
		       (setq this-pitch-and-family 2))
		   (if (not (memq lc-char '(lc-vn-1 lc-vn-2)))
		       nil
		     (let (ccl-vn-2 (x-get-ccl lc-vn-2))
		       (cond ((null ccl-vn-2)
			      (x-set-ccl lc-vn-2 ccl-win32-vn-2-viscii))
			     ((not (eq ccl-vn-2 ccl-win32-vn-2-viscii))
			      (x-set-ccl lc-vn-2 nil)
			      (x-set-ccl lc-vn-2 ccl-win32-vn-2-viscii))))))
		  ;;  for Mule Bitmap Fonts or
		  ;;      Bitmap Fonts making from ETL BDF.
		  ((or (string-match " sMBF[0-9]+$" tmp-facename)
		       (string-match " sETL[0-9]+$" tmp-facename))
		   (or this-pitch-and-family
		       (setq this-pitch-and-family 17))
		   (setcar (nthcdr 3 tmp-logfont) 'symbol) )
		  ;; for MuleTrueType MultiByte Fonts.
		  ((string-match " jMTF$" tmp-facename)
		   (setcar (nthcdr 3 tmp-logfont) 'Japanese) )
		  (t
		   (let ((tmp-alist (assq lc-char win32-lc-char-to-lf-charset-alist)))
		     (setcar (nthcdr 3 tmp-logfont)
			     (cond (tmp-alist
				    (cdr tmp-alist))
				   ((eq lc-char 'lc-kr)
				    (if (= win32-user-code-page 1361)
					'iso-2022-kr
				      'Korean))
				   (t
				    'ansi))))))))
	;; pitch-and-family
	(setcar (nthcdr 4 tmp-logfont)
		(if this-pitch-and-family
		    this-pitch-and-family
		  49))
	(win32-set-fontset-font fontset-name lc-char font-name tmp-logfont)
	(while lc-group
	  (let* ((lc-char2 (car lc-group))
		 (lc-name2 (symbol-name lc-char2)))
	    (or (win32-fontset-font-exist-p lc-char2 fontset-name)
		(win32-set-fontset-font fontset-name lc-char2
					(concat (substring lc-name2 3 (length lc-name2))
						"-"
						(int-to-string height))
					tmp-logfont)))
	  (setq lc-group (cdr lc-group))))
      (setq font-list (cdr font-list)))
    ))

(defun win32-default-fontset (&optional size)
  (let (fonts fontset)
    (cond ((= size 12)
	   (setq fonts default-fonts-12
		 fontset "default-12"))
	  ((= size 14)
	   (setq fonts default-fonts-14
		 fontset "default-14"))
	  (t
	   nil))
    (if fontset
	(win32-new-fontset fontset fonts size))))

(defun win32-set-fontset-IMEfont (&optional fontset width height font)
  (let (tmp)
    (or (stringp fontset)
	(setq fontset (cdr (assq 'fontset (frame-parameters)))))
    (or (stringp font)
	(and (setq tmp (assoc fontset win32-IMEfont-alist))
	     (setq font (cdr tmp))))
    (and font fontset
	 (let ((ajust-width (car (win32-get-fontset-attribute fontset)))
	       (logfont (win32-get-font-property font 0 fontset)))
	   (setcar (nthcdr 1 logfont)
		   (if (integerp width)
		       width 0))
	   (if (integerp height)
	       nil
	     (setq height (* ajust-width 2)))
	   (setcar (nthcdr 2 logfont) height)
	   (win32-change-font-property font 4
				       logfont
				       fontset)))))

(defun win32-set-IMEfonts ()
  (let ((olg-fontset (list (assq 'fontset (frame-parameters)))))
    (while win32-IMEfont-alist
      (let* ((fontset (car (car win32-IMEfont-alist)))
	     (font (cdr (car win32-IMEfont-alist))))
	(modify-frame-parameters (selected-frame) (list (cons 'fontset fontset)))
	(win32-set-fontset-IMEfont fontset nil nil font))
      (setq win32-IMEfont-alist (cdr win32-IMEfont-alist)))
    (modify-frame-parameters (selected-frame) olg-fontset)))
;; end of patch

;(set-clipboard-coding-system *sjis*dos)
;(set-win32-system-coding-system *sjis*dos)

(defvar Mule-for-Win32-Version-String
  (purecopy "
Mule 2.3@19.30 for Win32  Version Alpha(,Fa(B) 0.05-13 of 1997.11.25
Presented Members of NIFTY-Serve FEXT."))

(defun Mule-for-Win32-Version ()
  Mule-for-Win32-Version-String)

(defalias 'font-list 'win32-font-list)
(defalias 'fontset-list 'win32-fontset-list)
(defalias 'x-selection-exists-p 'win32-clipboard-data-exist-p)

;(global-set-key "\C-cw" 'copy-region-to-clipboard)
;(global-set-key "\C-c\C-w" 'kill-region-to-clipboard)
;(global-set-key "\C-c\C-y" 'yank-from-clipboard)

;;; winnt.el ends here
